#!/usr/bin/env python
# coding: utf-8
import logging



logger = logging.getLogger('logger')
logger.setLevel(logging.INFO)
sh = logging.StreamHandler()
sh.setLevel(logging.INFO)
fmt = logging.Formatter(
    '[%(levelname)s] %(asctime)s %(filename)s[line:%(lineno)d] %(message)s')
sh.setFormatter(fmt)
logger.addHandler(sh)



#!/usr/bin/env python
# coding: utf-8
import sys
import os
import commands

__author__ = 'Huang xiongbiao(billo@qq.com)'


def run_GUI():
    checkRoot()
    try:
        stat, terminalId = commands.getstatusoutput('xdotool getactivewindow')
        from GUI import TickeysApp
        if stat == 0:
            TickeysApp(terminalId).run()
        else:
            TickeysApp().run()

    except Exception, e:
        print "Run GUI Fail, use CLI instead..Fail msg:%s" % str(e)
        run_CLI()


def run_CLI():
    checkRoot()
    from CLI import CLI
    CLI().cmdloop()


def checkRoot():
    print "Root checking..."
    if os.getegid() != 0:
        print "This program must be run as root.."
        sys.exit(0)
    print "Root checking success.."


def run():
    if len(sys.argv) != 2 or sys.argv[1] not in ['-c', '-g']:
        print "Usage: python run.py -c     ---CLI mode"
        print "       python run.py -g     ---GUI mode"
        if len(sys.argv) == 1:
            run_GUI()
        sys.exit(0)
    if sys.argv[1] == '-g':
        del sys.argv[1]  # otherwise kivy would regard it as option
        run_GUI()
    else:
        run_CLI()

if __name__ == '__main__':
    run()



#!/usr/bin/env python
# coding: utf-8
from pygame import mixer
from logger import logger
import json
import os

__author__ = 'Huang xiongbiao(billo@qq.com)'


class SoundPlayer():

    def __init__(self):
        try:
            os.chdir(os.path.dirname(__file__))
        except Exception:
            pass
        self.mixer = mixer
        self.mixer.init(frequency=22050)
        self.config = dict(
            (i['name'], i)
            for i in json.load(file('./Resources/data/schemes.json')))
        self.style = 'bubble'
        self.soundFileList = self.config[self.style]['files']
        self.key_audio_map = self.config[self.style]['key_audio_map']
        self.non_unique_count = self.config[self.style]['non_unique_count']
        self.soundEffectCache = []
        self.volume = 1.0
        self.pitch = 1.0
        self.changing = False
        self.cacheSoundEffect()

    # preload sound effect
    def cacheSoundEffect(self):
        try:
            self.changing = True
            self.mixer.quit()
            self.mixer.init(frequency=int(self.pitch*22050))
            self.soundEffectCache = []
            for effectFile in self.soundFileList:
                soundFile = './Resources/data/%s/%s' % \
                    (self.style, effectFile)
                logger.debug('Load sound file:' + soundFile)
                self.soundEffectCache.append(self.mixer.Sound(soundFile))
            self.setVolume(self.volume)

        except Exception, e:
            logger.error('Load sound files fail:' + str(e))
        finally:
            self.changing = False

    def setStyle(self, style):
        try:
            if self.style == style:
                return
            self.style = style
            self.soundFileList = self.config[self.style]['files']
            self.key_audio_map = self.config[self.style]['key_audio_map']
            self.non_unique_count = self.config[self.style]['non_unique_count']
            self.cacheSoundEffect()
        except Exception, e:
            logger.error(e)

    def setVolume(self, volume):
        self.volume = volume
        for soundEffect in self.soundEffectCache:
            soundEffect.set_volume(self.volume)

    # May raise nonfluency when init the mixer
    def setPitch(self, pitch):
        # if abs(self.mixer.get_init()[0]/22050.0 - pitch) < 0.1:
        #     return
        # it's weird
        pitch = 0.65 + 0.3 * pitch if pitch < 1 else pitch + 0.9
        self.pitch = pitch
        self.cacheSoundEffect()

    def play(self, key):

        if not self.key_audio_map.get(str(key)):
            self.key_audio_map[str(key)] = key % self.non_unique_count

        # avoid the situation that play sound while caching
        while self.changing:
            continue

        self.soundEffectCache[self.key_audio_map[str(key)]].play()
        # dir(self.soundEffectCache[self.key_audio_map[str(key)]])



#!/usr/bin/env python
# coding: utf-8
import os
import threading
from evdev import InputDevice
from select import select
from SoundPlayer import SoundPlayer
from logger import logger
import commands

__author__ = 'Huang xiongbiao(billo@qq.com)'

# input device file path
deviceFilePath = '/sys/class/input/'


class KeyboardHandler():

    def __init__(self):
        logger.debug("Keyboard deteccter created.")
        self.keyboardList = self.findKeyboard()
        self.threads = []
        self.inputRecord = []
        self.hotKey = [16, 30, 44, 2, 3, 4]  # QAZ123
        self.sp = SoundPlayer()
        self.GUIID = None

    # list all event's name and its device
    def showDevice(self):
        # os.chdir(deviceFilePath)
        for i in os.listdir(deviceFilePath):
            namePath = deviceFilePath + i + '/device/name'
            if os.path.isfile(namePath):
                logger.info("Name: %s Device: %s" % (i, file(namePath).read()))

    def setStyle(self, style):
        self.sp.setStyle(style)

    def setVolume(self, volume):
        self.sp.setVolume(volume)

    def setPitch(self, pitch):
        self.sp.setPitch(pitch)

    @property
    def GUIID(self):
        return self.GUIID

    def showGUI(self):
        if not self.GUIID:
            return
        commands.getstatusoutput(
            "xdotool windowactivate --sync %s" % self.GUIID)

    # return with a list of keyboard's event
    def findKeyboard(self):
        # os.chdir(deviceFilePath)
        keyboardList = []
        for event in os.listdir(deviceFilePath):
            namePath = deviceFilePath + event + '/device/name'
            isFind = os.path.isfile(namePath) and \
                file(namePath).read().lower().find('keyboard') != -1
            if isFind:
                keyboardList.append(event)

        try:
            assert len(keyboardList) >= 1
        except AssertionError:
            logger.error("Keyborad Not Found!!")
            keyboardList = ['event4']

        return keyboardList

    def detectInputKey(self, eventName):
        dev = InputDevice('/dev/input/' + eventName)
        while True:
            select([dev], [], [])
            for event in dev.read():
                if (event.value == 1 or event.value == 0) and event.code != 0:
                    if event.value == 1:
                        self.sp.play(event.code)
                        self.checkShowWindow(event.code)
                    logger.debug(
                        "Key: %s Status: %s" %
                        (event.code, "pressed" if event.value else "release"))

    def checkShowWindow(self, keycode):
        if len(self.inputRecord) > 0 and keycode == self.inputRecord[-1]:
            return
        if keycode == self.hotKey[len(self.inputRecord)]:
            self.inputRecord.append(keycode)
            logger.debug(self.inputRecord)
            if len(self.inputRecord) == 6:
                self.showGUI()
                self.inputRecord = []
        else:
            self.inputRecord = []

    def startDetecting(self):

        for i in self.keyboardList:
            t = threading.Thread(target=self.detectInputKey, args=(i,))
            self.threads.append(t)
            t.start()

    def stopDetecting(self):
        for t in self.threads:
            t._Thread__stop()

if __name__ == '__main__':
    detecter = KeyboardHandler()
    detecter.startDetecting()



#!/usr/bin/env python
# coding: utf-8
from kivy.app import App
from kivy.uix.spinner import Spinner
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.gridlayout import GridLayout
from KeyboardHandler import KeyboardHandler
from kivy.lang import Builder
from StartupHandler import add_startup_linux, check_startup_file, delete_startup_linux
from __init__ import __version__

import sys
import commands
import webbrowser
import os
reload(sys)
sys.setdefaultencoding("utf-8")

Builder.load_string('''
<Main>:
    pos: 0,0
    padding: 50
    rows: 6
    row_force_default: True
    row_default_height: 50
    spacing: 50
    orientation: 'vertical'

    canvas:
        Color:
            rgb: 0.42, 0.42, 0.42, 1
        Rectangle:
            pos: 0,0
            size: self.size
    Label:
        bold: True
        text: 'Tickeys'
        font_size: 42
        size_hint_y: None

    SpinnerRow
    AdjustVol
    AdjustPitch
    ExitAndSwitchRow

    InforRow


<AdjustVol>
    Label:
        bold: True
        color: 1, 1, 1, 1
        font_size: 25
        size_hint_x: None
        width: 250
        text: 'Vol:'
    Slider:
        min: 0.0
        max: 1.0
        value: 1.0
        width: 300
        on_value: root.setVolume(self.value)

<AdjustPitch>
    Label:
        bold: True
        color: 1, 1, 1, 1
        font_size: 25
        size_hint_x: None
        width: 250
        text: 'Pitch:'
    Slider:
        min: 0.0
        max: 3.0
        value: 1.0
        width: 300
        on_value: root.setPitch(self.value)


<SpinnerRow>:
    Label:
        bold: True
        color: 1, 1, 1, 1
        font_size: 25
        size_hint_x: None
        text: "Sound Effect:"
        width: 250
    EffectSpinner:
        on_text: root.change_style()


<EffectSpinner>:
    bold: True
    font_size: 25
    text: 'bubble'
    background_color: 2, 2, 2, 1
    color: 1, 1, 1, 1
    values:['bubble', 'mechanical', 'sword', 'typewriter',]

<ExitAndSwitchRow>:
    Label:
        size_hint_x: None
        width: root.width/6.0 - 120
    Label:
        size_hint_x: None
        color: 1, 1, 1, 1
        font_size: 17
        width: root.width/6.0 + 60
        text: 'Open at startup:'
    Switch:
        size_hint_x: None
        width: 40
        id: switcher
        active: root.have_added
        on_active: root.add_delete_startup_file(self.active)
    Label:
        size_hint_x: None
        width: root.width/6.0 - 35
    Button:
        size_hint_x: None
        width: 150
        background_color: 2, 2, 2, 1
        bold: True
        text: "EXIT"
        color: 0,0,0,1
        on_press: root.Exit()



<InforRow>:
    Label:
        color: 0.7, 0.7, 0.7, 1
        font_size: 23
        size_hint_x: None
        text: root.getVersion()
        width: root.width/3.0
    Label:
        color: 0.8, 0.8, 0.8, 1
        font_size: 20
        size_hint_x: None
        markup: True
        text: "[ref=->website]Project website[/ref]"
        width: root.width/3.0
        on_ref_press:root.openProjectWebsite()
    Label:
        color: 0.8, 0.8, 0.8, 1
        font_size: 18
        size_hint_x: None
        text: "Author: Bill (billo@qq.com)"
        width: root.width/3.0
        border: 1,1,1,1
        on_touch_move:
'''.encode('utf-8'))


class EffectSpinner(Spinner):
    pass


class SpinnerRow(BoxLayout):
    def change_style(self):
        self.parent.detecter.setStyle(self.children[0].text)


class AdjustVol(BoxLayout):
    def setVolume(self, volume):
        self.parent.detecter.setVolume(volume)


class AdjustPitch(BoxLayout):
    def setPitch(self, pitch):
        self.parent.detecter.setPitch(pitch)


class SwitcherRow(BoxLayout):
    pass


class ExitAndSwitchRow(BoxLayout):
    def Exit(self):
        self.parent.Exit()

    def add_delete_startup_file(self, active):
        if active:
            add_startup_linux()
        else:
            delete_startup_linux()

    @property
    def have_added(self):
        return check_startup_file()


class InforRow(BoxLayout):
    def openProjectWebsite(self):
        webbrowser.open_new("http://www.yingdev.com/projects/tickeys")

    def getVersion(self):
        return 'Version: '+__version__


class Main(GridLayout):
    def __init__(self, *args, **kwargs):
        super(Main, self).__init__(**kwargs)
        self.terminalId = args[0] if args else None
        self.GUIID = None
        # tool works preget
        if self.terminalId:
            stat, GUIID = commands.getstatusoutput('xdotool getactivewindow')
            if stat == 0:
                self.GUIID = GUIID
            # hide itself
                commands.getstatusoutput(
                    'xdotool getactivewindow windowminimize')
        self.detecter = KeyboardHandler()
        self.detecter.startDetecting()
        self.detecter.GUIID = self.GUIID
        self.hideTerminal()

    # @property
    # def detecter(self):
    #     return self.detecter

    def hideTerminal(self):
        if not self.terminalId:
            return
        commands.getstatusoutput(
            "xdotool windowactivate --sync %s" % self.terminalId)
        commands.getstatusoutput(
            "xdotool getactivewindow windowunmap")
        # if want to show terminal use windowminimize

    def Exit(self):
        self.detecter.stopDetecting()
        # Show the terminal
        # if self.terminalId:
        #     commands.getstatusoutput(
        #    "xdotool windowactivate --sync %s" % self.terminalId)
        #     commands.getstatusoutput(
        #    "xdotool getactivewindow windowmap")
        sys.exit(0)


class TickeysApp(App):
    def __init__(self, *args, **kwargs):
        super(TickeysApp, self).__init__(**kwargs)
        self.terminalId = args[0] if args else None

    def build(self):
        root = Main(self.terminalId)
        self.icon = 'tickeys.png'
        return root

    def on_stop(self):
        self.root.Exit()


if __name__ == '__main__':
    TickeysApp().run()
